package cz.muni.fi.pa165.core.facade.airplane;

import cz.muni.fi.pa165.core.exceptions.ResourceNotFoundException;
import cz.muni.fi.pa165.core.mapper.AirplaneMapper;
import cz.muni.fi.pa165.core.model.AirplaneDto;
import cz.muni.fi.pa165.core.model.NewAirplaneDtoRequest;
import cz.muni.fi.pa165.core.service.airplane.AirplaneService;
import cz.muni.fi.pa165.core.service.airplanetype.AirplaneTypeService;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
public class AirplaneFacadeImpl implements AirplaneFacade<Long> {

    private final AirplaneService airplaneService;
    private final AirplaneTypeService airplaneTypeService;
    private final AirplaneMapper airplaneMapper;

    public AirplaneFacadeImpl(AirplaneService airplaneService, AirplaneTypeService airplaneTypeService,
                              AirplaneMapper airplaneMapper) {
        this.airplaneService = airplaneService;
        this.airplaneTypeService = airplaneTypeService;
        this.airplaneMapper = airplaneMapper;
    }

    @Override
    public AirplaneDto save(NewAirplaneDtoRequest newAirplaneDtoRequest) {
        var entityToSave = airplaneMapper.toEntityFromNewRequest(newAirplaneDtoRequest);
        var savedEntity = airplaneService.save(entityToSave);
        return airplaneMapper.toDto(savedEntity);
    }

    @Override
    public Optional<AirplaneDto> findById(Long id) {
        var foundEntity = airplaneService.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("Not Found."));

        return Optional.ofNullable(airplaneMapper.toDto(foundEntity));
    }

    @Override
    public List<AirplaneDto> findAll() {
        var entities = airplaneService.findAll();
        return entities.stream()
                .map(airplaneMapper::toDto)
                .toList();
    }

    @Override
    public void deleteById(Long id) {
        airplaneService.deleteById(id);
    }

    @Override
    public void deleteAll() {
        airplaneService.deleteAll();
    }

    @Override
    public AirplaneDto assignAirplaneType(Long airplaneId, Long airplaneTypeId) {
        var airplaneEntityOpt = airplaneService.findById(airplaneId);
        var airplaneTypeEntityOpt = airplaneTypeService.findById(airplaneTypeId);

        if (airplaneEntityOpt.isEmpty() || airplaneTypeEntityOpt.isEmpty()) {
            throw new ResourceNotFoundException("Not Found.");
        }

        var airplaneEntity = airplaneEntityOpt.get();
        var airplaneTypeEntity = airplaneTypeEntityOpt.get();

        airplaneEntity.setType(airplaneTypeEntity);
        airplaneService.save(airplaneEntity);

        return airplaneMapper.toDto(airplaneEntity);
    }

    @Override
    public void deleteAirplaneTypeAssignment(Long airplaneId, Long airplaneTypeId) {
        var airplaneEntity = airplaneService.findById(airplaneId);
        var airplaneTypeEntity = airplaneTypeService.findById(airplaneTypeId);

        if (airplaneEntity.isPresent() && airplaneTypeEntity.isPresent()) {
            airplaneEntity.get().setType(null);
            airplaneService.save(airplaneEntity.get());
        }
    }

    @Override
    public AirplaneDto update(Long id, NewAirplaneDtoRequest newAirplaneDtoRequest) {
        var newAirplaneEntity = airplaneMapper.toEntityFromNewRequest(newAirplaneDtoRequest);

        return airplaneMapper.toDto(newAirplaneEntity);
    }

}
